import random

def move_only(organism, organism_list, world):
    """
    Only move.
    """
    return 'move'

def move_and_reproduce(organism, organism_list, world):
    """
    Move and reproduce. Reproduction occurs with probability 1/8.
    """
    choice = random.randint(0, 8)
    if choice == 0:
        return 'reproduce'
    else:
        return 'move'

def move_and_drink(organism, organism_list, world):
    """
    Move and drink. Each occurs with probability 1/2.
    """
    choice = random.randint(0, 8)
    if choice < 4:
        return 'drink'
    else:
        return 'move'

def move_reproduce_drink(organism, organism_list, world):
    """
    Move, drink, and reproduce. Reproduction occurs with probability 1/8.
    Drinks with probability 3/8, and moves with probability 1/2.
    """
    choice = random.randint(0, 8)
    if choice == 0:
        return 'reproduce'
    elif choice < 4:
        return 'drink'
    else:
        return 'move'



"""
Built-in dictionaries with world, species, and organism parameters (or fields).

Both Organism and World objects are initialized based on these field dictionaries,
and values are either populated from parameter files or take on the default
values specified in this module.
"""

world_field_names = {'dimensionality': 1,
                     'world_size': [10],
                     'initial_environment_dict': None,
                     'water': None,
                     'food': None,
                     'obstacles': None}

specific_organism_field_names = {'organism_id': None,
                        'dna': '0000',
                        'age': 0,
                        'alive': True,
                        'age_at_death': None,
                        'cause_of_death': None,
                        'position': [0],
                        'sex': None,
                        'ancestry': [],
                        'custom_methods_fns': None,
                        'water_current': None,
                        'time_without_water': 0,
                        'food_current': None,
                        'time_without_food': 0}

species_field_names = {'species_name': 'species1',
                        'movement_type': 'stationary',
                        'reproduction_type': None,
                        'drinking_type': None,
                        'eating_type': None,
                        'action_type': 'move_only',
                        'dna_length': 4,
                        'max_age': 20,
                        'max_time_without_food': None,
                        'max_time_without_water': None,
                        'mutation_rate': None,
                        'food_capacity': None,
                        'food_initial': None,
                        'food_metabolism': None,
                        'food_intake': None,
                        'water_capacity': None,
                        'water_initial': None,
                        'water_metabolism': None,
                        'water_intake': None}

# Combine organism and species field names
organism_field_names = dict(specific_organism_field_names, **species_field_names)



import os
import glob
import json
import random

import fields
from world import World
from organism import Organism

def parse(intent_list, organism_list):
    """
    Determine whether the intent list is valid and fix it otherwise.

    Parameters
    ----------
    intent_list : list of Organisms
        List of organisms with proposed organism states,
        after each organism has 'acted'
    organism_list : list of Organisms
        List of current organisms

    Returns
    -------
    updated_list : list of Organisms
        List of updated organisms with conflicts between intent_list and
        organism_list resolved.
    """
    # TODO: Figure out exactly how this should be controlled -- on the scale of
    # the universe, the world, or the organisms itself
    updated_list = []
    for organism in intent_list:
        if organism.age_at_death is None or organism.age_at_death == organism.age:
            updated_list.append(organism)
    return updated_list



import sys
import random

def stationary(organism, organism_list, world):
    """
    Organism stays still.
    """
    return [organism]

def simple_random(organism, organism_list, world):
    """
    Move in random direction with equal probability. For 2D, organisms
    don't walk diagonally.
    """
    position = organism.position
    size = world.world_size

    if len(size) == 1:
        [x] = position
        choice = random.randint(0, 2)
        if choice == 0 and x != 0:
            x -= 1
        elif choice == 1 and x != size[0] - 1:
            x += 1
        else:
            pass
        organism.position = [x]
    elif len(size) == 2:
        [x, y] = position
        x_choice = random.randint(0, 2)
        y_choice = random.randint(0, 2)
        if x_choice == 0 and x != 0:
            x -= 1
        elif x_choice == 1 and x != size[0] - 1:
            x += 1
        else:
            pass
        if y_choice == 0 and y != 0:
            y -= 1
        elif y_choice == 1 and y != size[1] - 1:
            y += 1
        else:
            pass
        organism.position = [x, y]
    else:
        sys.exit('Invalid world dimensionality!')
    return [organism]



import sys
import random

def constant_drink(organism, organism_list, world):
    """
    Intake constant amount of water from world if water is present.
    """
    position = organism.position
    size = world.world_size

    if len(size) == 1:
        if world.water[organism.position[0]] > 0:
            # Actually intake water
            organism.water_current += organism.water_intake

            # for one dimension
            world.water[organism.position[0]] -= organism.water_intake

    # organism.water_current -= organism.water_metabolism
    # if organism.water_current > organism.water_capacity:
    #     organism.water_current = organism.water_capacity
    # if organism.water_current <= 0:
    #     organism.water_current = 0
    #     organism.time_without_water += 1
    # else if organism.time_without_water > 0:
    #     organism.time_without_water = 0

    return [organism]



"""
blossom is a package for simulating evolution
"""

__version__ = "0.1"

import sys
import os
sys.path.append(os.path.dirname(__file__))

from universe import Universe
from organism import Organism
from world import World
from fields import world_field_names, \
    specific_organism_field_names, species_field_names, \
    organism_field_names
import dataset_io
import parameter_io

import organism_behavior
from world_generator import *
